"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var fs = _interopRequireWildcard(require("fs-extra"));

var path = _interopRequireWildcard(require("path"));

var _changeCase = require("change-case");

var _prompts = _interopRequireDefault(require("prompts"));

var _getModularRoot = _interopRequireDefault(require("./utils/getModularRoot"));

var _execAsync = _interopRequireDefault(require("./utils/execAsync"));

var logger = _interopRequireWildcard(require("./utils/logger"));

var _actionPreflightCheck = _interopRequireDefault(require("./utils/actionPreflightCheck"));

var _getAllFiles = _interopRequireDefault(require("./utils/getAllFiles"));

var _LineFilterOutStream = _interopRequireDefault(require("./utils/LineFilterOutStream"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const packagesRoot = 'packages';

async function addPackage({
  destination,
  type: typeArg,
  name: nameArg,
  template: templateNameArg,
  preferOffline = true,
  verbose = false
}) {
  var _modularTemplatePacka, _modularTemplatePacka2, _subprocess$stderr;

  const {
    type: packageType,
    name
  } = (typeArg && nameArg ? {
    type: typeArg,
    name: nameArg
  } : null) || (await (0, _prompts.default)([{
    name: 'name',
    type: 'text',
    message: `What would you like to name this package?`,
    initial: (0, _changeCase.paramCase)(destination)
  }, {
    name: 'type',
    type: 'select',
    message: `What kind of package is this?`,
    choices: [{
      title: 'A plain package',
      value: 'package'
    }, {
      title: 'A view within an application',
      value: 'view'
    }, {
      title: 'A standalone application',
      value: 'app'
    }, {
      title: 'Choose my own template',
      value: '__CHOOSE_MY_OWN__'
    }],
    initial: 0
  }]));
  let templateName = templateNameArg || packageType;

  if (packageType === '__CHOOSE_MY_OWN__') {
    logger.warn('You are choosing to install a template which is not maintained by the modular team.');
    const typeResponse = await (0, _prompts.default)([{
      name: 'typeName',
      type: 'text',
      message: `Which modular template would you like to use.?`
    }]);
    const typeName = typeResponse.typeName;

    if (typeName[0] === '@') {
      const [scope, typePackageName] = typeName.split('/');
      templateName = `${scope}/modular-template-${typePackageName}`;
    }
  } else {
    templateName = `modular-template-${templateName}`;
  }

  const modularRoot = (0, _getModularRoot.default)();
  const newComponentName = (0, _changeCase.pascalCase)(name);
  const newPackagePath = path.join(modularRoot, packagesRoot, destination);

  if (fs.existsSync(newPackagePath)) {
    throw new Error(`A package already exists at ${destination}!`);
  } // try and find the modular template packge, if it's already been installed
  // in the project then continue without needing to do an install.
  // else we will fetch it from the yarn registry.


  try {
    require.resolve(`${templateName}/package.json`);
  } catch (e) {
    var _templateInstallSubpr;

    const templateInstallSubprocess = (0, _execAsync.default)('yarnpkg', ['add', templateName, '--prefer-offline', '--silent', '-W'], {
      cwd: modularRoot,
      stderr: 'pipe'
    }); // Remove warnings

    (_templateInstallSubpr = templateInstallSubprocess.stderr) === null || _templateInstallSubpr === void 0 ? void 0 : _templateInstallSubpr.pipe(new _LineFilterOutStream.default(/.*warning.*/));

    if (verbose) {
      var _templateInstallSubpr2;

      (_templateInstallSubpr2 = templateInstallSubprocess.stderr) === null || _templateInstallSubpr2 === void 0 ? void 0 : _templateInstallSubpr2.pipe(process.stderr);
    }

    await templateInstallSubprocess;
  }

  const newModularPackageJsonPath = require.resolve(`${templateName}/package.json`);

  const modularTemplatePackageJson = await fs.readJSON(newModularPackageJsonPath);
  const modularType = modularTemplatePackageJson === null || modularTemplatePackageJson === void 0 ? void 0 : (_modularTemplatePacka = modularTemplatePackageJson.modular) === null || _modularTemplatePacka === void 0 ? void 0 : _modularTemplatePacka.type;

  if (modularType !== 'template') {
    throw new Error(`${templateName} is not marked as a "template". Check the package you are trying to install.`);
  }

  const modularTemplateType = modularTemplatePackageJson === null || modularTemplatePackageJson === void 0 ? void 0 : (_modularTemplatePacka2 = modularTemplatePackageJson.modular) === null || _modularTemplatePacka2 === void 0 ? void 0 : _modularTemplatePacka2.templateType;

  if (!['app', 'view', 'package'].includes(modularTemplateType)) {
    throw new Error(`${templateName} has modular type: ${modularTemplateType}, which does not exist, please use update this template`);
  }

  const packageTypePath = path.dirname(newModularPackageJsonPath); // create a new package source folder

  await fs.mkdirp(newPackagePath);
  await fs.copy(packageTypePath, newPackagePath, {
    recursive: true,

    filter(src) {
      return !(path.basename(src) === 'package.json');
    }

  });
  const packageFilePaths = (0, _getAllFiles.default)(newPackagePath);

  for (const packageFilePath of packageFilePaths) {
    fs.writeFileSync(packageFilePath, fs.readFileSync(packageFilePath, 'utf8').replace(/PackageName__/g, name).replace(/ComponentName__/g, newComponentName));
  }

  await fs.writeJson(path.join(newPackagePath, 'package.json'), {
    name,
    private: modularTemplateType === 'app',
    modular: {
      type: modularTemplateType
    },
    main: modularTemplatePackageJson.main,
    dependencies: modularTemplatePackageJson.dependencies,
    version: '1.0.0'
  }, {
    spaces: 2
  });

  if (modularTemplateType === 'app') {
    // add a tsconfig, because CRA expects it
    await fs.writeJSON(path.join(newPackagePath, 'tsconfig.json'), {
      extends: path.relative(newPackagePath, modularRoot) + '/tsconfig.json'
    }, {
      spaces: 2
    });
  }

  const yarnArgs = verbose ? ['--verbose'] : ['--silent'];

  if (preferOffline) {
    yarnArgs.push('--prefer-offline');
  }

  const subprocess = (0, _execAsync.default)('yarnpkg', yarnArgs, {
    cwd: modularRoot,
    stderr: 'pipe'
  }); // Remove warnings

  (_subprocess$stderr = subprocess.stderr) === null || _subprocess$stderr === void 0 ? void 0 : _subprocess$stderr.pipe(new _LineFilterOutStream.default(/.*warning.*/));

  if (verbose) {
    var _subprocess$stderr2;

    (_subprocess$stderr2 = subprocess.stderr) === null || _subprocess$stderr2 === void 0 ? void 0 : _subprocess$stderr2.pipe(process.stderr);
  }

  await subprocess;
}

var _default = (0, _actionPreflightCheck.default)(addPackage);

exports.default = _default;
//# sourceMappingURL=addPackage.js.map